package areacodes;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

public class AreaCodeImpl implements AreaCodeService
{
    private HashMap<String, AreaCode> map = new HashMap<String, AreaCode>();

    public AreaCodeImpl()
    {
        String line = null;
        String areaCodeString = null;
        StringTokenizer st;
     
         try
         {
        	 String path = "data/AreaCodes.txt";
        	 BufferedReader in = 
        			 new BufferedReader(
        					 new   InputStreamReader(this.getClass()
        							 .getClassLoader().getResourceAsStream(path))); 
            while (true)
            {
                line = in.readLine();
                if (line == null)
                {
                    break;
                }
                st = new StringTokenizer(line, "#");
                AreaCode code = new AreaCode();
                areaCodeString = st.nextToken().trim();           
                code.setAreaCode(Integer.parseInt(areaCodeString));
                code.setState(st.nextToken());
                code.setOffset(st.nextToken());
                code.setDescription(st.nextToken());

               map.put(areaCodeString, code);
            }
            in.close();
        }
        catch(IOException ioError)
        {
            ioError.printStackTrace();
        }
        catch(NumberFormatException numberError)
        {
            numberError.printStackTrace();
        }
    }

        public String getState(int areaCode) 
        {
            String key = Integer.toString(areaCode);
            AreaCode code = (AreaCode) map.get(key);
            if (code == null)
            {
                return null;
            }
            return code.getState();
        }
        
        public String getDescription(int areaCode)
        	throws AreaCodeException 
        {
            String key = Integer.toString(areaCode);
            AreaCode code = (AreaCode) map.get(key);
            if (code == null)
            {
                throw new AreaCodeException("No Description");
            }          
            return code.getDescription();
        }
        
        public String getOffset(int areaCode)
        {
            String key = Integer.toString(areaCode);
            AreaCode code = (AreaCode) map.get(key);
            if (code == null)
            {
                return null;
            }         
            return code.getOffset();
        }
        
        public AreaCode getAreaCode(int areaCode)
        {
        	String key = Integer.toString(areaCode);
            AreaCode code = (AreaCode) map.get(key);        
            return code;
        }

		@Override
		public String[] findAreaCodesByState(String state)
		{
			List<String> results = new ArrayList<String>();
			for (Map.Entry<String,AreaCode> entry : map.entrySet())  
			{
				AreaCode current = entry.getValue();
				if (state.equalsIgnoreCase(current.getState()))
				{
					results.add(Integer.toString(current.getAreaCode()));
				}
			}
			String[] output = results.stream().toArray(String[] ::new); 
			return output;
		} 

		@Override
		public AreaCode findByCode(int code)
		{
			String key = Integer.toString(code);
			return map.get(key);
		}

		@Override
		public AreaCode[] findByOffset(double offset)
		{
			List<AreaCode> results = new ArrayList<AreaCode>();
			for (Map.Entry<String,AreaCode> entry : map.entrySet())  
			{
				AreaCode current = entry.getValue();
				try 
				{
		            Double field = Double.parseDouble(current.getOffset().trim());
		            if (offset == field)
					{
						results.add(current);
					}
		        } 
				catch (NumberFormatException e) 
				{
		            continue;
		        }	
			}
			AreaCode[] output = results.stream().toArray(AreaCode[] ::new); 
			return output;
		}

		@Override
		public boolean validate(AreaCode input)
		{
			if(map.containsValue(input))
			{
				return true;
			}
			else
			{
				return false;
			}
		}

		@Override
		public AreaCode[] findInSameState(AreaCode input)
		{
			String state = input.getState();
			List<AreaCode> results = new ArrayList<AreaCode>();
			for (Map.Entry<String,AreaCode> entry : map.entrySet())  
			{
				AreaCode current = entry.getValue();
				if (state.equalsIgnoreCase(current.getState()))
				{
					results.add(current);
				}
			}
			AreaCode[] output = results.stream().toArray(AreaCode[] ::new); 
			return output;
		}
}
